@@ -4,9 +4,9 @@ import unittest
 def suite():
     modules_to_test = (
         # core utilities
-        'base.historyarray',
-        'base.attributes', 
         'base.dependency',
+        'base.utils',
+        'base.except',
         )
     alltests = unittest.TestSuite()
     for name in modules_to_test:
@@ -18,4 +18,4 @@ def suite():
 
 
 if __name__ == '__main__':
-    testbase.runTests(suite())
+    testbase.main(suite())
