@@ -49,7 +49,7 @@ class testcase(PersistTest):
         class Preferences(ActiveMapper):
             class mapping:
                 __table__        = 'preferences'
-                favorite_color   = column(String)
+                favorite_color   = column(String(128))
                 personality_type = column(String)
 
         class Address(ActiveMapper):
@@ -63,7 +63,7 @@ class testcase(PersistTest):
                 address_1   = column(String)
                 city        = column(String)
                 state       = column(String)
-                postal_code = column(String)
+                postal_code = column(String(128))
                 person_id   = column(Integer, foreign_key=ForeignKey('person.id'))
 
         activemapper.metadata.connect(testbase.db)
@@ -237,6 +237,7 @@ class testcase(PersistTest):
         # uses a function which I dont think existed when you first wrote ActiveMapper.
         p1 = self.create_person_one()
         self.assertEquals(p1.preferences.person, p1)
+        objectstore.flush()
         objectstore.delete(p1)
         
         objectstore.flush()
