@@ -1,17 +1,19 @@
-from testbase import PersistTest
-import sqlalchemy.util as util
-import unittest
 import testbase
-from sqlalchemy import *
-from sqlalchemy.ext.associationproxy import *
 
-db = testbase.db
+from sqlalchemy import *
+from sqlalchemy.orm import *
+from sqlalchemy.orm.collections import collection
+from sqlalchemy.ext.associationproxy import *
+from testlib import *
+
 
 class DictCollection(dict):
+    @collection.appender
     def append(self, obj):
         self[obj.foo] = obj
-    def __iter__(self):
-        return self.itervalues()
+    @collection.remover
+    def remove(self, obj):
+        del self[obj.foo]
 
 class SetCollection(set):
     pass
@@ -22,19 +24,21 @@ class ListCollection(list):
 class ObjectCollection(object):
     def __init__(self):
         self.values = list()
+    @collection.appender
     def append(self, obj):
         self.values.append(obj)
+    @collection.remover
+    def remove(self, obj):
+        self.values.remove(obj)
     def __iter__(self):
         return iter(self.values)
-    def clear(self):
-        self.values.clear()
 
 class _CollectionOperations(PersistTest):
     def setUp(self):
         collection_class = self.collection_class
 
-        metadata = BoundMetaData(db)
-    
+        metadata = MetaData(testbase.db)
+
         parents_table = Table('Parent', metadata,
                               Column('id', Integer, primary_key=True),
                               Column('name', String))
@@ -47,7 +51,7 @@ class _CollectionOperations(PersistTest):
 
         class Parent(object):
             children = association_proxy('_children', 'name')
-        
+
             def __init__(self, name):
                 self.name = name
 
@@ -75,7 +79,8 @@ class _CollectionOperations(PersistTest):
         self.metadata.drop_all()
 
     def roundtrip(self, obj):
-        self.session.save(obj)
+        if obj not in self.session:
+            self.session.save(obj)
         self.session.flush()
         id, type_ = obj.id, type(obj)
         self.session.clear()
@@ -83,7 +88,7 @@ class _CollectionOperations(PersistTest):
 
     def _test_sequence_ops(self):
         Parent, Child = self.Parent, self.Child
-        
+
         p1 = Parent('P1')
 
         self.assert_(not p1._children)
@@ -109,7 +114,7 @@ class _CollectionOperations(PersistTest):
 
         self.assert_(p1._children[0].name == 'regular')
         self.assert_(p1._children[1].name == 'proxied')
-    
+
         del p1._children[1]
 
         self.assert_(len(p1._children) == 1)
@@ -120,7 +125,7 @@ class _CollectionOperations(PersistTest):
 
         self.assert_(len(p1._children) == 0)
         self.assert_(len(p1.children) == 0)
-    
+
         p1.children = ['a','b','c']
         self.assert_(len(p1._children) == 3)
         self.assert_(len(p1.children) == 3)
@@ -130,7 +135,61 @@ class _CollectionOperations(PersistTest):
 
         self.assert_(len(p1._children) == 3)
         self.assert_(len(p1.children) == 3)
-        
+
+        popped = p1.children.pop()
+        self.assert_(len(p1.children) == 2)
+        self.assert_(popped not in p1.children)
+        p1 = self.roundtrip(p1)
+        self.assert_(len(p1.children) == 2)
+        self.assert_(popped not in p1.children)
+
+        p1.children[1] = 'changed-in-place'
+        self.assert_(p1.children[1] == 'changed-in-place')
+        inplace_id = p1._children[1].id
+        p1 = self.roundtrip(p1)
+        self.assert_(p1.children[1] == 'changed-in-place')
+        assert p1._children[1].id == inplace_id
+
+        p1.children.append('changed-in-place')
+        self.assert_(p1.children.count('changed-in-place') == 2)
+
+        p1.children.remove('changed-in-place')
+        self.assert_(p1.children.count('changed-in-place') == 1)
+
+        p1 = self.roundtrip(p1)
+        self.assert_(p1.children.count('changed-in-place') == 1)
+
+        p1._children = []
+        self.assert_(len(p1.children) == 0)
+
+        after = ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j']
+        p1.children = ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j']
+        self.assert_(len(p1.children) == 10)
+        self.assert_([c.name for c in p1._children] == after)
+
+        p1.children[2:6] = ['x'] * 4
+        after = ['a', 'b', 'x', 'x', 'x', 'x', 'g', 'h', 'i', 'j']
+        self.assert_(p1.children == after)
+        self.assert_([c.name for c in p1._children] == after)
+
+        p1.children[2:6] = ['y']
+        after = ['a', 'b', 'y', 'g', 'h', 'i', 'j']
+        self.assert_(p1.children == after)
+        self.assert_([c.name for c in p1._children] == after)
+
+        p1.children[2:3] = ['z'] * 4
+        after = ['a', 'b', 'z', 'z', 'z', 'z', 'g', 'h', 'i', 'j']
+        self.assert_(p1.children == after)
+        self.assert_([c.name for c in p1._children] == after)
+
+        p1.children[2::2] = ['O'] * 4
+        after = ['a', 'b', 'O', 'z', 'O', 'z', 'O', 'h', 'O', 'j']
+        self.assert_(p1.children == after)
+        self.assert_([c.name for c in p1._children] == after)
+
+        self.assertRaises(TypeError, set, [p1.children])
+
+
 class DefaultTest(_CollectionOperations):
     def __init__(self, *args, **kw):
         super(DefaultTest, self).__init__(*args, **kw)
@@ -139,6 +198,7 @@ class DefaultTest(_CollectionOperations):
     def test_sequence_ops(self):
         self._test_sequence_ops()
 
+
 class ListTest(_CollectionOperations):
     def __init__(self, *args, **kw):
         super(ListTest, self).__init__(*args, **kw)
@@ -192,7 +252,7 @@ class CustomDictTest(DictTest):
 
         self.assert_(p1._children['a'].name == 'regular')
         self.assert_(p1._children['b'].name == 'proxied')
-    
+
         del p1._children['b']
 
         self.assert_(len(p1._children) == 1)
@@ -208,11 +268,37 @@ class CustomDictTest(DictTest):
         self.assert_(len(p1._children) == 3)
         self.assert_(len(p1.children) == 3)
 
+        self.assert_(set(p1.children) == set(['d','e','f']))
+
         del ch
         p1 = self.roundtrip(p1)
         self.assert_(len(p1._children) == 3)
         self.assert_(len(p1.children) == 3)
-    
+
+        p1.children['e'] = 'changed-in-place' 
+        self.assert_(p1.children['e'] == 'changed-in-place')
+        inplace_id = p1._children['e'].id
+        p1 = self.roundtrip(p1)
+        self.assert_(p1.children['e'] == 'changed-in-place')
+        self.assert_(p1._children['e'].id == inplace_id)
+
+        p1._children = {}
+        self.assert_(len(p1.children) == 0)
+
+        try:
+            p1._children = []
+            self.assert_(False)
+        except exceptions.ArgumentError:
+            self.assert_(True)
+
+        try:
+            p1._children = None
+            self.assert_(False)
+        except exceptions.ArgumentError:
+            self.assert_(True)
+
+        self.assertRaises(TypeError, set, [p1.children])
+
 
 class SetTest(_CollectionOperations):
     def __init__(self, *args, **kw):
@@ -228,7 +314,7 @@ class SetTest(_CollectionOperations):
         self.assert_(not p1.children)
 
         ch1 = Child('regular')
-        p1._children.append(ch1)
+        p1._children.add(ch1)
 
         self.assert_(ch1 in p1._children)
         self.assert_(len(p1._children) == 1)
@@ -245,7 +331,8 @@ class SetTest(_CollectionOperations):
         self.assert_(len(p1.children) == 2)
         self.assert_(len(p1._children) == 2)
 
-        self.assert_(set([o.name for o in p1._children]) == set(['regular', 'proxied']))
+        self.assert_(set([o.name for o in p1._children]) ==
+                     set(['regular', 'proxied']))
 
         ch2 = None
         for o in p1._children:
@@ -263,7 +350,7 @@ class SetTest(_CollectionOperations):
 
         self.assert_(len(p1._children) == 0)
         self.assert_(len(p1.children) == 0)
-    
+
         p1.children = ['a','b','c']
         self.assert_(len(p1._children) == 3)
         self.assert_(len(p1.children) == 3)
@@ -298,11 +385,11 @@ class SetTest(_CollectionOperations):
         p1 = self.roundtrip(p1)
         self.assert_(len(p1.children) == 2)
         self.assert_(popped not in p1.children)
-    
+
         p1.children = ['a','b','c']
         p1 = self.roundtrip(p1)
         self.assert_(p1.children == set(['a','b','c']))
-    
+
         p1.children.discard('b')
         p1 = self.roundtrip(p1)
         self.assert_(p1.children == set(['a', 'c']))
@@ -311,6 +398,24 @@ class SetTest(_CollectionOperations):
         p1 = self.roundtrip(p1)
         self.assert_(p1.children == set(['c']))
 
+        p1._children = set()
+        self.assert_(len(p1.children) == 0)
+
+        try:
+            p1._children = []
+            self.assert_(False)
+        except exceptions.ArgumentError:
+            self.assert_(True)
+
+        try:
+            p1._children = None
+            self.assert_(False)
+        except exceptions.ArgumentError:
+            self.assert_(True)
+
+        self.assertRaises(TypeError, set, [p1.children])
+
+
     def test_set_comparisons(self):
         Parent, Child = self.Parent, self.Child
 
@@ -337,7 +442,7 @@ class SetTest(_CollectionOperations):
                              control.issubset(other))
             self.assertEqual(p1.children.issuperset(other),
                              control.issuperset(other))
-            
+
             self.assert_((p1.children == other)  ==  (control == other))
             self.assert_((p1.children != other)  ==  (control != other))
             self.assert_((p1.children < other)   ==  (control < other))
@@ -379,14 +484,7 @@ class SetTest(_CollectionOperations):
                         print 'want', repr(control)
                         print 'got', repr(p.children)
                         raise
-    
-    # workaround for bug #548
-    def test_set_pop(self):
-        Parent, Child = self.Parent, self.Child
-        p = Parent('p1')
-        p.children.add('a')
-        p.children.pop()
-        self.assert_(True)
+
 
 class CustomSetTest(SetTest):
     def __init__(self, *args, **kw):
@@ -420,8 +518,8 @@ class CustomObjectTest(_CollectionOperations):
 
 class ScalarTest(PersistTest):
     def test_scalar_proxy(self):
-        metadata = BoundMetaData(db)
-    
+        metadata = MetaData(testbase.db)
+
         parents_table = Table('Parent', metadata,
                               Column('id', Integer, primary_key=True),
                               Column('name', String))
@@ -439,7 +537,7 @@ class ScalarTest(PersistTest):
                                     creator=lambda v: Child(bar=v))
             baz = association_proxy('child', 'baz',
                                     creator=lambda v: Child(baz=v))
-        
+
             def __init__(self, name):
                 self.name = name
 
@@ -457,12 +555,13 @@ class ScalarTest(PersistTest):
         session = create_session()
 
         def roundtrip(obj):
-            session.save(obj)
+            if obj not in session:
+                session.save(obj)
             session.flush()
             id, type_ = obj.id, type(obj)
             session.clear()
             return session.query(type_).get(id)
-            
+
         p = Parent('p')
 
         # No child
@@ -482,7 +581,7 @@ class ScalarTest(PersistTest):
         self.assert_(p.foo == 'a')
         self.assert_(p.bar == 'x')
         self.assert_(p.baz == 'c')
-        
+
         p = roundtrip(p)
 
         self.assert_(p.foo == 'a')
@@ -532,12 +631,12 @@ class ScalarTest(PersistTest):
         # Ensure an immediate __set__ works.
         p2 = Parent('p2')
         p2.bar = 'quux'
-        
+
 
 class LazyLoadTest(PersistTest):
     def setUp(self):
-        metadata = BoundMetaData(db)
-    
+        metadata = MetaData(testbase.db)
+
         parents_table = Table('Parent', metadata,
                               Column('id', Integer, primary_key=True),
                               Column('name', String))
@@ -550,7 +649,7 @@ class LazyLoadTest(PersistTest):
 
         class Parent(object):
             children = association_proxy('_children', 'name')
-        
+
             def __init__(self, name):
                 self.name = name
 
@@ -592,7 +691,7 @@ class LazyLoadTest(PersistTest):
         # Is there a better way to ensure that the association_proxy
         # didn't convert a lazy load to an eager load?  This does work though.
         self.assert_('_children' not in p.__dict__)
-        self.assert_(len(p._children.data) == 3)
+        self.assert_(len(p._children) == 3)
         self.assert_('_children' in p.__dict__)
 
     def test_eager_list(self):
@@ -608,7 +707,7 @@ class LazyLoadTest(PersistTest):
         p = self.roundtrip(p)
 
         self.assert_('_children' in p.__dict__)
-        self.assert_(len(p._children.data) == 3)
+        self.assert_(len(p._children) == 3)
 
     def test_lazy_scalar(self):
         Parent, Child = self.Parent, self.Child
@@ -639,7 +738,7 @@ class LazyLoadTest(PersistTest):
 
         self.assert_('_children' in p.__dict__)
         self.assert_(p._children is not None)
-    
+
 
 if __name__ == "__main__":
-    testbase.main()        
+    testbase.main()
