@@ -58,7 +58,7 @@ class CollectionsTest(PersistTest):
         canary = Canary()
         manager.register_class(Foo)
         manager.register_attribute(Foo, 'attr', True, extension=canary,
-                                   typecallable=typecallable)
+                                   typecallable=typecallable, useobject=True)
 
         obj = Foo()
         adapter = collections.collection_adapter(obj.attr)
@@ -96,7 +96,7 @@ class CollectionsTest(PersistTest):
         canary = Canary()
         manager.register_class(Foo)
         manager.register_attribute(Foo, 'attr', True, extension=canary,
-                                   typecallable=typecallable)
+                                   typecallable=typecallable, useobject=True)
 
         obj = Foo()
         adapter = collections.collection_adapter(obj.attr)
@@ -238,7 +238,7 @@ class CollectionsTest(PersistTest):
         canary = Canary()
         manager.register_class(Foo)
         manager.register_attribute(Foo, 'attr', True, extension=canary,
-                                   typecallable=typecallable)
+                                   typecallable=typecallable, useobject=True)
 
         obj = Foo()
         direct = obj.attr
@@ -257,7 +257,7 @@ class CollectionsTest(PersistTest):
         self.assert_(set(obj.attr) == set([e2]))
         self.assert_(e1 in canary.removed)
         self.assert_(e2 in canary.added)
- 
+
         e3 = creator()
         real_list = [e3]
         obj.attr = real_list
@@ -265,7 +265,7 @@ class CollectionsTest(PersistTest):
         self.assert_(set(obj.attr) == set([e3]))
         self.assert_(e2 in canary.removed)
         self.assert_(e3 in canary.added)
-       
+
         e4 = creator()
         try:
             obj.attr = set([e4])
@@ -274,6 +274,21 @@ class CollectionsTest(PersistTest):
             self.assert_(e4 not in canary.data)
             self.assert_(e3 in canary.data)
 
+        e5 = creator()
+        e6 = creator()
+        e7 = creator()
+        obj.attr = [e5, e6, e7]
+        self.assert_(e5 in canary.added)
+        self.assert_(e6 in canary.added)
+        self.assert_(e7 in canary.added)
+
+        obj.attr = [e6, e7]
+        self.assert_(e5 in canary.removed)
+        self.assert_(e6 in canary.added)
+        self.assert_(e7 in canary.added)
+        self.assert_(e6 not in canary.removed)
+        self.assert_(e7 not in canary.removed)
+
     def test_list(self):
         self._test_adapter(list)
         self._test_list(list)
@@ -347,7 +362,7 @@ class CollectionsTest(PersistTest):
         canary = Canary()
         manager.register_class(Foo)
         manager.register_attribute(Foo, 'attr', True, extension=canary,
-                                   typecallable=typecallable)
+                                   typecallable=typecallable, useobject=True)
 
         obj = Foo()
         adapter = collections.collection_adapter(obj.attr)
@@ -480,7 +495,7 @@ class CollectionsTest(PersistTest):
         canary = Canary()
         manager.register_class(Foo)
         manager.register_attribute(Foo, 'attr', True, extension=canary,
-                                   typecallable=typecallable)
+                                   typecallable=typecallable, useobject=True)
 
         obj = Foo()
         direct = obj.attr
@@ -585,7 +600,7 @@ class CollectionsTest(PersistTest):
         canary = Canary()
         manager.register_class(Foo)
         manager.register_attribute(Foo, 'attr', True, extension=canary,
-                                   typecallable=typecallable)
+                                   typecallable=typecallable, useobject=True)
 
         obj = Foo()
         adapter = collections.collection_adapter(obj.attr)
@@ -703,7 +718,7 @@ class CollectionsTest(PersistTest):
         canary = Canary()
         manager.register_class(Foo)
         manager.register_attribute(Foo, 'attr', True, extension=canary,
-                                   typecallable=typecallable)
+                                   typecallable=typecallable, useobject=True)
 
         obj = Foo()
         direct = obj.attr
@@ -878,7 +893,7 @@ class CollectionsTest(PersistTest):
         canary = Canary()
         manager.register_class(Foo)
         manager.register_attribute(Foo, 'attr', True, extension=canary,
-                                   typecallable=typecallable)
+                                   typecallable=typecallable, useobject=True)
 
         obj = Foo()
         adapter = collections.collection_adapter(obj.attr)
@@ -1012,7 +1027,7 @@ class CollectionsTest(PersistTest):
         canary = Canary()
         manager.register_class(Foo)
         manager.register_attribute(Foo, 'attr', True, extension=canary,
-                                   typecallable=Custom)
+                                   typecallable=Custom, useobject=True)
 
         obj = Foo()
         adapter = collections.collection_adapter(obj.attr)
@@ -1081,7 +1096,7 @@ class CollectionsTest(PersistTest):
         canary = Canary()
         creator = entity_maker
         manager.register_class(Foo)
-        manager.register_attribute(Foo, 'attr', True, extension=canary)
+        manager.register_attribute(Foo, 'attr', True, extension=canary, useobject=True)
 
         obj = Foo()
         col1 = obj.attr
@@ -1155,7 +1170,6 @@ class DictHelpersTest(ORMTest):
         collections.collection_adapter(p.children).append_with_event(
             Child('foo', 'newvalue'))
         
-        session.save(p)
         session.flush()
         session.clear()
         
@@ -1215,7 +1229,6 @@ class DictHelpersTest(ORMTest):
         collections.collection_adapter(p.children).append_with_event(
             Child('foo', '1', 'newvalue'))
         
-        session.save(p)
         session.flush()
         session.clear()
         
