@@ -6,6 +6,7 @@ from sqlalchemy.sql import compiler
 from sqlalchemy.engine import default
 from sqlalchemy.orm import *
 from testlib import *
+from testlib import engines
 from testlib.fixtures import *
 
 class QueryTest(FixtureTest):
@@ -14,13 +15,8 @@ class QueryTest(FixtureTest):
     
     def setUpAll(self):
         super(QueryTest, self).setUpAll()
-        install_fixture_data()
         self.setup_mappers()
-    
-    def tearDownAll(self):
-        clear_mappers()
-        super(QueryTest, self).tearDownAll()
-          
+
     def setup_mappers(self):
         mapper(User, users, properties={
             'addresses':relation(Address, backref='user'),
@@ -38,10 +34,10 @@ class QueryTest(FixtureTest):
 
 class UnicodeSchemaTest(QueryTest):
     keep_mappers = False
-    
+
     def setup_mappers(self):
         pass
-        
+
     def define_tables(self, metadata):
         super(UnicodeSchemaTest, self).define_tables(metadata)
         global uni_meta, uni_users
@@ -49,11 +45,11 @@ class UnicodeSchemaTest(QueryTest):
         uni_users = Table(u'users', uni_meta,
             Column(u'id', Integer, primary_key=True),
             Column(u'name', String(30), nullable=False))
-            
+
     def test_get(self):
         mapper(User, uni_users)
         assert User(id=7) == create_session(bind=testbase.db).query(User).get(7)
-        
+
 class GetTest(QueryTest):
     def test_get(self):
         s = create_session()
@@ -73,54 +69,60 @@ class GetTest(QueryTest):
         print s.primary_key
         print m.primary_key
         assert s.primary_key == m.primary_key
-        
+
         row = s.select(use_labels=True).execute().fetchone()
         print row[s.primary_key[0]]
-         
+
         sess = create_session()
         assert sess.query(SomeUser).get(7).name == 'jack'
 
     def test_load(self):
         s = create_session()
-        
+
         try:
             assert s.query(User).load(19) is None
             assert False
         except exceptions.InvalidRequestError:
             assert True
-            
+
         u = s.query(User).load(7)
         u2 = s.query(User).load(7)
         assert u is u2
         s.clear()
         u2 = s.query(User).load(7)
         assert u is not u2
-        
+
         u2.name = 'some name'
         a = Address(email_address='some other name')
         u2.addresses.append(a)
         assert u2 in s.dirty
         assert a in u2.addresses
-        
+
         s.query(User).load(7)
         assert u2 not in s.dirty
         assert u2.name =='jack'
         assert a not in u2.addresses
-        
-    @testing.exclude('mysql', '<', (5, 0))  # fixme
+
+    @testing.exclude('mysql', '<', (4, 1))
     def test_unicode(self):
-        """test that Query.get properly sets up the type for the bind parameter.  using unicode would normally fail 
+        """test that Query.get properly sets up the type for the bind parameter.  using unicode would normally fail
         on postgres, mysql and oracle unless it is converted to an encoded string"""
-        
-        table = Table('unicode_data', users.metadata, 
+
+        metadata = MetaData(engines.utf8_engine())
+        table = Table('unicode_data', metadata,
             Column('id', Unicode(40), primary_key=True),
             Column('data', Unicode(40)))
-        table.create()
-        ustring = 'petit voix m\xe2\x80\x99a '.decode('utf-8')
-        table.insert().execute(id=ustring, data=ustring)
-        class LocalFoo(Base):pass
-        mapper(LocalFoo, table)
-        assert create_session().query(LocalFoo).get(ustring) == LocalFoo(id=ustring, data=ustring)
+        try:
+            metadata.create_all()
+            ustring = 'petit voix m\xe2\x80\x99a'.decode('utf-8')
+            table.insert().execute(id=ustring, data=ustring)
+            class LocalFoo(Base):
+                pass
+            mapper(LocalFoo, table)
+            self.assertEquals(create_session().query(LocalFoo).get(ustring),
+                              LocalFoo(id=ustring, data=ustring))
+        finally:
+            metadata.drop_all()
 
     def test_populate_existing(self):
         s = create_session()
@@ -148,19 +150,19 @@ class GetTest(QueryTest):
         s.query(User).populate_existing().all()
         assert u.addresses[0].email_address == 'lala'
         assert u.orders[1].items[2].description == 'item 12'
-        
+
         # eager load does
         s.query(User).options(eagerload('addresses'), eagerload_all('orders.items')).populate_existing().all()
         assert u.addresses[0].email_address == 'jack@bean.com'
         assert u.orders[1].items[2].description == 'item 5'
-        
+
 class OperatorTest(QueryTest):
     """test sql.Comparator implementation for MapperProperties"""
-    
+
     def _test(self, clause, expected):
         c = str(clause.compile(dialect = default.DefaultDialect()))
         assert c == expected, "%s != %s" % (c, expected)
-        
+
     def test_arithmetic(self):
         create_session().query(User)
         for (py_op, sql_op) in ((operator.add, '+'), (operator.mul, '*'),
@@ -206,7 +208,10 @@ class OperatorTest(QueryTest):
                 self.assert_(compiled == fwd_sql or compiled == rev_sql,
                              "\n'" + compiled + "'\n does not match\n'" +
                              fwd_sql + "'\n or\n'" + rev_sql + "'")
-    
+
+    def test_op(self):
+        assert str(User.name.op('ilike')('17').compile(dialect=default.DefaultDialect())) == "users.name ilike :users_name"
+        
     def test_in(self):
          self._test(User.id.in_(['a', 'b']),
                     "users.id IN (:users_id, :users_id_1)")
@@ -225,24 +230,24 @@ class OperatorTest(QueryTest):
         ):
             c = expr.compile(dialect=default.DefaultDialect())
             assert str(c) == compare, "%s != %s" % (str(c), compare)
-            
-            
+
+
 class CompileTest(QueryTest):
     def test_deferred(self):
         session = create_session()
         s = session.query(User).filter(and_(addresses.c.email_address == bindparam('emailad'), Address.user_id==User.id)).compile()
-        
+
         l = session.query(User).instances(s.execute(emailad = 'jack@bean.com'))
         assert [User(id=7)] == l
-    
+
 class SliceTest(QueryTest):
     def test_first(self):
         assert  User(id=7) == create_session().query(User).first()
-        
+
         assert create_session().query(User).filter(User.id==27).first() is None
-        
+
         # more slice tests are available in test/orm/generative.py
-        
+
 class TextTest(QueryTest):
     def test_fulltext(self):
         assert [User(id=7), User(id=8), User(id=9),User(id=10)] == create_session().query(User).from_statement("select * from users").all()
@@ -256,24 +261,25 @@ class TextTest(QueryTest):
 
     def test_binds(self):
         assert [User(id=8), User(id=9)] == create_session().query(User).filter("id in (:id1, :id2)").params(id1=8, id2=9).all()
-        
+
 class FilterTest(QueryTest):
     def test_basic(self):
         assert [User(id=7), User(id=8), User(id=9),User(id=10)] == create_session().query(User).all()
 
+    @testing.fails_on('maxdb')
     def test_limit(self):
         assert [User(id=8), User(id=9)] == create_session().query(User).limit(2).offset(1).all()
 
         assert [User(id=8), User(id=9)] == list(create_session().query(User)[1:3])
 
         assert User(id=8) == create_session().query(User)[1]
-        
+
     def test_onefilter(self):
         assert [User(id=8), User(id=9)] == create_session().query(User).filter(User.name.endswith('ed')).all()
 
     def test_contains(self):
         """test comparing a collection to an object instance."""
-        
+
         sess = create_session()
         address = sess.query(Address).get(3)
         assert [User(id=8)] == sess.query(User).filter(User.addresses.contains(address)).all()
@@ -291,9 +297,9 @@ class FilterTest(QueryTest):
             assert False
         except exceptions.InvalidRequestError:
             assert True
-            
+
         #assert [User(id=7), User(id=9), User(id=10)] == sess.query(User).filter(User.addresses!=address).all()
-    
+
     def test_any(self):
         sess = create_session()
 
@@ -305,15 +311,16 @@ class FilterTest(QueryTest):
             filter(User.addresses.any(id=4)).all()
 
         assert [User(id=9)] == sess.query(User).filter(User.addresses.any(email_address='fred@fred.com')).all()
-    
+
+    @testing.unsupported('maxdb') # can core
     def test_has(self):
         sess = create_session()
         assert [Address(id=5)] == sess.query(Address).filter(Address.user.has(name='fred')).all()
-        
+
         assert [Address(id=2), Address(id=3), Address(id=4), Address(id=5)] == sess.query(Address).filter(Address.user.has(User.name.like('%ed%'))).all()
-        
+
         assert [Address(id=2), Address(id=3), Address(id=4)] == sess.query(Address).filter(Address.user.has(User.name.like('%ed%'), id=8)).all()
-            
+
     def test_contains_m2m(self):
         sess = create_session()
         item = sess.query(Item).get(3)
@@ -323,7 +330,7 @@ class FilterTest(QueryTest):
 
     def test_comparison(self):
         """test scalar comparison to an object instance"""
-        
+
         sess = create_session()
         user = sess.query(User).get(8)
         assert [Address(id=2), Address(id=3), Address(id=4)] == sess.query(Address).filter(Address.user==user).all()
@@ -339,14 +346,49 @@ class AggregateTest(QueryTest):
     def test_apply(self):
         sess = create_session()
         assert sess.query(Order).apply_sum(Order.user_id * Order.address_id).filter(Order.id.in_([2, 3, 4])).one() == 79
+
+    def test_having(self):
+        sess = create_session()
+        assert [User(name=u'ed',id=8)] == sess.query(User).group_by([c for c in User.c]).join('addresses').having(func.count(Address.c.id)> 2).all()
+
+        assert [User(name=u'jack',id=7), User(name=u'fred',id=9)] == sess.query(User).group_by([c for c in User.c]).join('addresses').having(func.count(Address.c.id)< 2).all()
         
-                
 class CountTest(QueryTest):
     def test_basic(self):
         assert 4 == create_session().query(User).count()
 
         assert 2 == create_session().query(User).filter(users.c.name.endswith('ed')).count()
 
+class DistinctTest(QueryTest):
+    def test_basic(self):
+        assert [User(id=7), User(id=8), User(id=9),User(id=10)] == create_session().query(User).distinct().all()
+        assert [User(id=7), User(id=9), User(id=8),User(id=10)] == create_session().query(User).distinct().order_by(desc(User.name)).all()
+
+    def test_joined(self):
+        """test that orderbys from a joined table get placed into the columns clause when DISTINCT is used"""
+        
+        sess = create_session()
+        q = sess.query(User).join('addresses').distinct().order_by(desc(Address.email_address))
+
+        assert [User(id=7), User(id=9), User(id=8)] == q.all()
+
+        sess.clear()
+        
+        # test that it works on embedded eagerload/LIMIT subquery 
+        q = sess.query(User).join('addresses').distinct().options(eagerload('addresses')).order_by(desc(Address.email_address)).limit(2)
+
+        def go():
+            assert [
+                User(id=7, addresses=[
+                    Address(id=1)
+                ]), 
+                User(id=9, addresses=[
+                    Address(id=5)
+                ]), 
+            ] == q.all()
+        self.assert_sql_count(testbase.db, go, 1)
+        
+    
 class TextTest(QueryTest):
     def test_fulltext(self):
         assert [User(id=7), User(id=8), User(id=9),User(id=10)] == create_session().query(User).from_statement("select * from users").all()
@@ -360,13 +402,13 @@ class TextTest(QueryTest):
 
     def test_binds(self):
         assert [User(id=8), User(id=9)] == create_session().query(User).filter("id in (:id1, :id2)").params(id1=8, id2=9).all()
-        
-        
+
+
 class ParentTest(QueryTest):
     def test_o2m(self):
         sess = create_session()
         q = sess.query(User)
-        
+
         u1 = q.filter_by(name='jack').one()
 
         # test auto-lookup of property
@@ -388,7 +430,7 @@ class ParentTest(QueryTest):
     def test_noparent(self):
         sess = create_session()
         q = sess.query(User)
-        
+
         u1 = q.filter_by(name='jack').one()
 
         try:
@@ -402,8 +444,8 @@ class ParentTest(QueryTest):
         i1 = sess.query(Item).filter_by(id=2).one()
         k = sess.query(Keyword).with_parent(i1).all()
         assert [Keyword(name='red'), Keyword(name='small'), Keyword(name='square')] == k
-        
-    
+
+
 class JoinTest(QueryTest):
 
     def test_overlapping_paths(self):
@@ -424,13 +466,13 @@ class JoinTest(QueryTest):
 
             result = create_session().query(User).outerjoin(['orders', 'items'], aliased=aliased).filter_by(id=3).reset_joinpoint().outerjoin(['orders','address'], aliased=aliased).filter_by(id=1).all()
             assert [User(id=7, name='jack')] == result
-    
+
     def test_overlap_with_aliases(self):
         oalias = orders.alias('oalias')
 
         result = create_session().query(User).select_from(users.join(oalias)).filter(oalias.c.description.in_(["order 1", "order 2", "order 3"])).join(['orders', 'items']).all()
         assert [User(id=7, name='jack'), User(id=9, name='fred')] == result
-        
+
         result = create_session().query(User).select_from(users.join(oalias)).filter(oalias.c.description.in_(["order 1", "order 2", "order 3"])).join(['orders', 'items']).filter_by(id=4).all()
         assert [User(id=7, name='jack')] == result
 
@@ -462,7 +504,7 @@ class JoinTest(QueryTest):
         q = sess.query(User).join('orders').filter(Order.description=="order 3").join(['orders', 'items']).filter(Order.description=="item 1")
         assert [] == q.all()
         assert q.count() == 0
-        
+
         q = sess.query(User).join('orders', aliased=True).filter(Order.items.any(Item.description=='item 4'))
         assert [User(id=7)] == q.all()
 
@@ -534,8 +576,8 @@ class MultiplePathTest(ORMTest):
 
         create_session().query(T1).join('t2s_1', aliased=True).filter(t2.c.id==5).reset_joinpoint().join('t2s_2').all()
         create_session().query(T1).join('t2s_1').filter(t2.c.id==5).reset_joinpoint().join('t2s_2', aliased=True).all()
-        
-        
+
+
 
 class SynonymTest(QueryTest):
     keep_mappers = True
@@ -578,25 +620,27 @@ class SynonymTest(QueryTest):
         ):
             sess = create_session()
             q = sess.query(User)
-        
+
             u1 = q.filter_by(**{nameprop:'jack'}).one()
 
             o = sess.query(Order).with_parent(u1, property=orderprop).all()
             assert [Order(description="order 1"), Order(description="order 3"), Order(description="order 5")] == o
-        
+
 class InstancesTest(QueryTest):
 
     def test_from_alias(self):
 
         query = users.select(users.c.id==7).union(users.select(users.c.id>7)).alias('ulist').outerjoin(addresses).select(use_labels=True,order_by=['ulist.id', addresses.c.id])
-        q = create_session().query(User)
+        sess =create_session()
+        q = sess.query(User)
 
         def go():
             l = q.options(contains_alias('ulist'), contains_eager('addresses')).instances(query.execute())
             assert fixtures.user_address_result == l
         self.assert_sql_count(testbase.db, go, 1)
 
-
+        sess.clear()
+        
         def go():
             l = q.options(contains_alias('ulist'), contains_eager('addresses')).from_statement(query).all()
             assert fixtures.user_address_result == l
@@ -605,13 +649,16 @@ class InstancesTest(QueryTest):
     def test_contains_eager(self):
 
         selectquery = users.outerjoin(addresses).select(users.c.id<10, use_labels=True, order_by=[users.c.id, addresses.c.id])
-        q = create_session().query(User)
+        sess = create_session()
+        q = sess.query(User)
 
         def go():
             l = q.options(contains_eager('addresses')).instances(selectquery.execute())
             assert fixtures.user_address_result[0:3] == l
         self.assert_sql_count(testbase.db, go, 1)
 
+        sess.clear()
+        
         def go():
             l = q.options(contains_eager('addresses')).from_statement(selectquery).all()
             assert fixtures.user_address_result[0:3] == l
@@ -620,20 +667,24 @@ class InstancesTest(QueryTest):
     def test_contains_eager_alias(self):
         adalias = addresses.alias('adalias')
         selectquery = users.outerjoin(adalias).select(use_labels=True, order_by=[users.c.id, adalias.c.id])
-        q = create_session().query(User)
+        sess = create_session()
+        q = sess.query(User)
 
         def go():
             # test using a string alias name
             l = q.options(contains_eager('addresses', alias="adalias")).instances(selectquery.execute())
             assert fixtures.user_address_result == l
         self.assert_sql_count(testbase.db, go, 1)
-
+        sess.clear()
+        
         def go():
             # test using the Alias object itself
             l = q.options(contains_eager('addresses', alias=adalias)).instances(selectquery.execute())
             assert fixtures.user_address_result == l
         self.assert_sql_count(testbase.db, go, 1)
-
+        
+        sess.clear()
+        
         def decorate(row):
             d = {}
             for c in addresses.columns:
@@ -645,12 +696,33 @@ class InstancesTest(QueryTest):
             l = q.options(contains_eager('addresses', decorator=decorate)).instances(selectquery.execute())
             assert fixtures.user_address_result == l
         self.assert_sql_count(testbase.db, go, 1)
+        sess.clear()
+        
+        oalias = orders.alias('o1')
+        ialias = items.alias('i1')
+        query = users.outerjoin(oalias).outerjoin(order_items).outerjoin(ialias).select(use_labels=True).order_by(users.c.id).order_by(oalias.c.id).order_by(ialias.c.id)
+        q = create_session().query(User)
+        # test using string alias with more than one level deep
+        def go():
+            l = q.options(contains_eager('orders', alias='o1'), contains_eager('orders.items', alias='i1')).instances(query.execute())
+            assert fixtures.user_order_result == l
+        self.assert_sql_count(testbase.db, go, 1)
+
+        sess.clear()
+        
+        # test using Alias with more than one level deep
+        def go():
+            l = q.options(contains_eager('orders', alias=oalias), contains_eager('orders.items', alias=ialias)).instances(query.execute())
+            assert fixtures.user_order_result == l
+        self.assert_sql_count(testbase.db, go, 1)
+        sess.clear()
+
 
     def test_multi_mappers(self):
-        sess = create_session()
 
-        (user7, user8, user9, user10) = sess.query(User).all()
-        (address1, address2, address3, address4, address5) = sess.query(Address).all()
+        test_session = create_session()
+        (user7, user8, user9, user10) = test_session.query(User).all()
+        (address1, address2, address3, address4, address5) = test_session.query(Address).all()
 
         # note the result is a cartesian product
         expected = [(user7, address1),
@@ -659,28 +731,37 @@ class InstancesTest(QueryTest):
             (user8, address4),
             (user9, address5),
             (user10, None)]
-        
+
+        sess = create_session()
+
         selectquery = users.outerjoin(addresses).select(use_labels=True, order_by=[users.c.id, addresses.c.id])
         q = sess.query(User)
         l = q.instances(selectquery.execute(), Address)
         assert l == expected
-
+        
+        sess.clear()
+        
         for aliased in (False, True):
             q = sess.query(User)
+
             q = q.add_entity(Address).outerjoin('addresses', aliased=aliased)
             l = q.all()
             assert l == expected
+            sess.clear()
 
             q = sess.query(User).add_entity(Address)
             l = q.join('addresses', aliased=aliased).filter_by(email_address='ed@bettyboop.com').all()
             assert l == [(user8, address3)]
-        
+            sess.clear()
+
             q = sess.query(User, Address).join('addresses', aliased=aliased).filter_by(email_address='ed@bettyboop.com')
             assert q.all() == [(user8, address3)]
+            sess.clear()
 
             q = sess.query(User, Address).join('addresses', aliased=aliased).options(eagerload('addresses')).filter_by(email_address='ed@bettyboop.com')
             assert q.all() == [(user8, address3)]
-
+            sess.clear()
+            
     def test_aliased_multi_mappers(self):
         sess = create_session()
 
@@ -694,38 +775,59 @@ class InstancesTest(QueryTest):
             (user8, address4),
             (user9, address5),
             (user10, None)]
-        
+
         q = sess.query(User)
         adalias = addresses.alias('adalias')
         q = q.add_entity(Address, alias=adalias).select_from(users.outerjoin(adalias))
         l = q.all()
         assert l == expected
 
+        sess.clear()
+        
         q = sess.query(User).add_entity(Address, alias=adalias)
         l = q.select_from(users.outerjoin(adalias)).filter(adalias.c.email_address=='ed@bettyboop.com').all()
         assert l == [(user8, address3)]
-        
+
     def test_multi_columns(self):
+        sess = create_session()
+
+        expected = [(u, u.name) for u in sess.query(User).all()]
+        
+        for add_col in (User.name, users.c.name, User.c.name):
+            assert sess.query(User).add_column(add_col).all() == expected
+            sess.clear()
+            
+        try:
+            sess.query(User).add_column(object()).all()
+            assert False
+        except exceptions.InvalidRequestError, e:
+            assert "Invalid column expression" in str(e)
+            
+        
+    def test_multi_columns_2(self):
         """test aliased/nonalised joins with the usage of add_column()"""
         sess = create_session()
+
         (user7, user8, user9, user10) = sess.query(User).all()
         expected = [(user7, 1),
             (user8, 3),
             (user9, 1),
             (user10, 0)
             ]
-        
+
         for aliased in (False, True):
             q = sess.query(User)
             q = q.group_by([c for c in users.c]).order_by(User.id).outerjoin('addresses', aliased=aliased).add_column(func.count(Address.id).label('count'))
             l = q.all()
             assert l == expected
-
+            sess.clear()
+            
         s = select([users, func.count(addresses.c.id).label('count')]).select_from(users.outerjoin(addresses)).group_by(*[c for c in users.c]).order_by(User.id)
         q = sess.query(User)
         l = q.add_column("count").from_statement(s).all()
         assert l == expected
-
+    
+        
     def test_two_columns(self):
         sess = create_session()
         (user7, user8, user9, user10) = sess.query(User).all()
@@ -741,21 +843,25 @@ class InstancesTest(QueryTest):
         l = q.add_column("count").add_column("concat").from_statement(s).all()
         assert l == expected
         
+        sess.clear()
+        
         # test with select_from()
         q = create_session().query(User).add_column(func.count(addresses.c.id))\
             .add_column(("Name:" + users.c.name)).select_from(users.outerjoin(addresses))\
             .group_by([c for c in users.c]).order_by(users.c.id)
-            
-        assert q.all() == expected
 
+        assert q.all() == expected
+        sess.clear()
+        
         # test with outerjoin() both aliased and non
         for aliased in (False, True):
             q = create_session().query(User).add_column(func.count(addresses.c.id))\
                 .add_column(("Name:" + users.c.name)).outerjoin('addresses', aliased=aliased)\
                 .group_by([c for c in users.c]).order_by(users.c.id)
-            
-            assert q.all() == expected
 
+            assert q.all() == expected
+            sess.clear()
+            
 class CustomJoinTest(QueryTest):
     keep_mappers = False
 
@@ -792,7 +898,7 @@ class SelfReferentialJoinTest(ORMTest):
                 self.children.append(node)
 
         mapper(Node, nodes, properties={
-            'children':relation(Node, lazy=True, join_depth=3, 
+            'children':relation(Node, lazy=True, join_depth=3,
                 backref=backref('parent', remote_side=[nodes.c.id])
             )
         })
@@ -807,7 +913,7 @@ class SelfReferentialJoinTest(ORMTest):
         sess.save(n1)
         sess.flush()
         sess.clear()
-        
+
         # TODO: the aliasing of the join in query._join_to has to limit the aliasing
         # among local_side / remote_side (add local_side as an attribute on PropertyLoader)
         # also implement this idea in EagerLoader
@@ -816,7 +922,7 @@ class SelfReferentialJoinTest(ORMTest):
 
         node = sess.query(Node).join(['children', 'children'], aliased=True).filter_by(data='n122').first()
         assert node.data=='n1'
-        
+
         node = sess.query(Node).filter_by(data='n122').join('parent', aliased=True).filter_by(data='n12').\
             join('parent', aliased=True, from_joinpoint=True).filter_by(data='n1').first()
         assert node.data == 'n122'
@@ -858,10 +964,10 @@ class ExternalColumnsTest(QueryTest):
 
         mapper(Address, addresses, properties={
             'user':relation(User, lazy=True)
-        })    
+        })
 
         sess = create_session()
-        l = sess.query(User).select()
+        l = sess.query(User).all()
         assert [
             User(id=7, concat=14, count=1),
             User(id=8, concat=16, count=3),
@@ -876,9 +982,9 @@ class ExternalColumnsTest(QueryTest):
             Address(id=4, user=User(id=8, concat=16, count=3)),
             Address(id=5, user=User(id=9, concat=18, count=1))
         ]
-        
+
         assert address_result == sess.query(Address).all()
-        
+
         # run the eager version twice to test caching of aliased clauses
         for x in range(2):
             sess.clear()
@@ -887,11 +993,13 @@ class ExternalColumnsTest(QueryTest):
             self.assert_sql_count(testbase.db, go, 1)
 
         tuple_address_result = [(address, address.user) for address in address_result]
-        
+
         tuple_address_result == sess.query(Address).join('user').add_entity(User).all()
 
         assert tuple_address_result == sess.query(Address).join('user', aliased=True, id='ualias').add_entity(User, id='ualias').all()
 
+
+
 if __name__ == '__main__':
     testbase.main()
 
